import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter/material.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:nacht/core/core.dart';
import 'package:nacht/widgets/widgets.dart';

import '../models/models.dart';
import '../providers/providers.dart';

class HomePage extends HookWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final currentIndex = useState(0);

    return LazyIndexedStack(
      initialIndex: 0,
      currentIndex: currentIndex.value,
      destinations: destinations.map((item) => item.builder).toList(),
      duration: kShortAnimationDuration,
      builder: (context, child, animation) {
        return Scaffold(
          body: AnimationProvider(
            animation: animation,
            child: child,
          ),
          bottomNavigationBar: HookConsumer(
            builder: (context, ref, child) {
              final controller = useAnimationController(
                initialValue: 1,
                duration: kShortAnimationDuration,
              );

              ref.listen<NavigationInfo>(
                navigationProvider,
                (prev, next) {
                  if (!next.forceHide && prev?.event != next.event) {
                    next.event.when(
                      delta: (delta) => controller.value -= delta * 0.01,
                      end: () {
                        if (controller.value >= 0.5) {
                          controller.forward();
                        } else {
                          controller.reverse();
                        }
                      },
                    );
                  }

                  final prevForceHide = prev?.forceHide ?? false;
                  if (next.forceHide && !prevForceHide) {
                    controller.reverse();
                  } else if (!next.forceHide && prevForceHide) {
                    controller.forward();
                  }
                },
              );

              return AnimatedBottomBar(
                controller: controller,
                child: NavigationBar(
                  selectedIndex: currentIndex.value,
                  onDestinationSelected: (index) => currentIndex.value = index,
                  destinations: List.generate(destinations.length, (index) {
                    final destination = destinations[index];
                    return NavigationDestination(
                      icon: destination.icon,
                      selectedIcon: destination.selectedIcon,
                      label: destination.label,
                    );
                  }),
                ),
              );
            },
          ),
        );
      },
    );
  }
}
